<?php

/**
 * @package     Joomla.Plugin
 * @subpackage  Captcha
 *
 * @copyright   Copyright (C) 2005 - 2014 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */
defined('_JEXEC') or die;

/**
 * HashCash Plugin.
 * Based on the official hashcash library( https://developers.google.com/hashcash/docs/php )
 *
 * @package     Joomla.Plugin
 * @subpackage  Captcha
 * @since       2.5
 */
class PlgCaptchaHashCash extends JPlugin {

    /**
     * Load the language file on instantiation.
     *
     * @var    boolean
     * @since  3.1
     */
    protected $autoloadLanguage = true;

    /**
     * Initialise the captcha
     *
     * @param   string  $id  The id of the field.
     *
     * @return  Boolean	True on success, false otherwise
     *
     * @since  2.5
     */
    public function onInit($id = 'dynamic_hashcash_1') {
        $document = JFactory::getDocument();
        $app = JFactory::getApplication();
        $app->getLanguage()->load('plg_captcha_hashcash', JPATH_ADMINISTRATOR);
        $session = $app->getSession();

        JHtml::_('jquery.framework');
	$min = JFactory::getConfig()->get('debug',false)?'':'.min';
        foreach(array('sha256','hashcash') as $script) {
            $document->addScript(JURI::root(true).'/media/plg_captcha_hashcash/'.$script.$min.'.js',array('version'=>'auto'));
        }
        $vars = array();
        $vars['remote_addr'] = $session->get('remote_addr',filter_input(INPUT_SERVER,'REMOTE_ADDR'),'plg_captcha_hashcash');
        $vars['level'] = (int)$this->params->get('difficulty',3);
        $vars['request_time'] = $session->get('request_time',time(),'plg_captcha_hashcash');
	$vars['detectheadless'] = (bool)$this->params->get('detectheadless',0);
        
        $session->set('remote_addr',$vars['remote_addr'],'plg_captcha_hashcash');
        $session->set('level',$vars['level'],'plg_captcha_hashcash');
        $session->set('request_time',$vars['request_time'],'plg_captcha_hashcash');        
        
        $script = 'jQuery(document).ready(function($){$("#'.$id.'").HashCash(Joomla.getOptions("HashCash"));});';
        $document->addScriptDeclaration($script);
	$document->addScriptOptions('HashCash',$vars);

        return true;
    }

    /**
     * Gets the challenge HTML
     *
     * @param   string  $name   The name of the field.
     * @param   string  $id     The id of the field.
     * @param   string  $class  The class of the field. This should be passed as
     *                          e.g. 'class="required"'.
     *
     * @return  string  The HTML to be embedded in the form.
     *
     * @since  2.5
     */
    public function onDisplay($name, $id = 'dynamic_hashcash_1', $class = '') {
        return '<input type="hidden" id="' . $id . '" data-name="'.$name.'" name="hashcash_response_field"'.$class.'>';
    }

    /**
     * Calls an HTTP POST function to verify if the user's guess was correct
     *
     * @param   string  $code  Answer provided by user.
     *
     * @return  True if the answer is correct, false otherwise
     *
     * @since  2.5
     */
    public function onCheckAnswer($code) {
        $session = JFactory::getSession();
        $input = JFactory::getApplication()->input;        
        
        $remote_addr = $session->get('remote_addr',filter_input(INPUT_SERVER,'REMOTE_ADDR'),'plg_captcha_hashcash');
        $level = (int)$this->params->get('difficulty',3);
        $request_time = $session->get('request_time',time(),'plg_captcha_hashcash');
        
        $count = $input->get('hashcash_response_field', '', 'string');
        
        // Discard spam submissions
        if ($count == null || strlen($count) == 0) {
            $this->_subject->setError(JText::_('PLG_CAPTCHA_HASHCASH_ERROR_EMPTY_RESPONSE'));
            return false;
        }
        
        $string = $remote_addr.$request_time.$count;
        $sha256 = hash('sha256',$string,false);
        $valid = preg_match('/^0{'.$level.'}/',$sha256);
        
        foreach(array('remote_addr','request_time') as $name) {
                $session->clear($name,'plg_captcha_hashcash');
        }

        if ($valid) {
            return true;
        } else {
            // @todo use exceptions here
            $this->_subject->setError(JText::_('PLG_CAPTCHA_HASHCASH_ERROR_INVALID_RESPONSE'));

            return false;
        }
    }

}
